const knex = require("knex");
const Promise = require("bluebird");
const fse = require("fs-extra");
const path = require("path");
const { applySchema } = require("./knex-tools/");

async function applyModuleSchema(appConfig, options = {}) {
  const knexSpecPath = path.join(
    appConfig.graphql.dataLibPath,
    "db/knex-spec.json"
  );
  const knexSpec = fse.readJSONSync(knexSpecPath);
  const db = knex(appConfig.db.app);

  const {
    seedFunction,
    finalizeSchema,
    cleanSchema,
    commonSchema,
    moduleSchema,
    applyTables,
    applyContraints,
    applyViews,
    log = () => {},
  } = options;

  let skipTables = !applyTables;
  let skipConstraints = !applyContraints;
  let skipViews = !applyViews;
  let skipCleanSchema = !cleanSchema;

  let opts = {
    ...options,
    log,
    cleanSchema,
    commonSchema,
    moduleSchema,
    applyTables,
    applyContraints,
    applyViews,
    skipTables,
    skipConstraints,
    skipViews,
    skipCleanSchema,
  };

  await applySchema(db, knexSpec, seedFunction, opts, appConfig);

  if (finalizeSchema) {
    log(`[ORADM-FINALIZE] Finalizing schema`);
    await finalizeSchema(db, opts, appConfig);
  }

  await db.destroy();
}

module.exports = { applyModuleSchema };
