const _ = require("lodash");
const njk = require("nunjucks");
const fse = require("fs-extra");
const path = require("path");
const pluralize = require("pluralize");
const nameFormats = require("../graphql-generator/name-formats");

const { generateDataLibTests } = require("../unit-test-generator");

const resolverMethodsTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/resolverMethods.njk"),
  "utf8"
);

// const entityMethodsTemplate = fse.readFileSync(
//   path.join(__dirname, "./templates/entityMethods.njk"),
//   "utf8"
// );

const entityAssetPaths = [
  path.join(__dirname, "./core-files/populateNestedObjectsForDelete.js"),
  path.join(__dirname, "./core-files/populateNestedObjectsForDelete.test.js"),
  path.join(__dirname, "./core-files/populateNestedObjectsForUpdate.js"),
  path.join(__dirname, "./core-files/populateNestedObjectsForUpdate.test.js"),
  path.join(__dirname, "./core-files/populateNestedObjectsForCreate.js"),
  path.join(__dirname, "./core-files/populateNestedObjectsForCreate.test.js"),
  path.join(__dirname, "./core-files/getCidReferences.js"),
  path.join(__dirname, "./core-files/getCidReferences.test.js"),
  path.join(__dirname, "./core-files/resolveCidReferences.js"),
  path.join(__dirname, "./core-files/resolveCidReferences.test.js"),
  path.join(__dirname, "./core-files/getNestedParentRecords.js"),
  path.join(__dirname, "./core-files/getNestedParentRecords.test.js"),
  path.join(__dirname, "./core-files/insertParentRecords.js"),
  path.join(__dirname, "./core-files/insertParentRecords.test.js"),
  path.join(__dirname, "./core-files/mapAttributesToColumns.js"),
  path.join(__dirname, "./core-files/mapAttributesToColumns.test.js"),
  path.join(__dirname, "./core-files/applyFilter.js"),
  path.join(__dirname, "./core-files/applyFilter.test.js"),
  path.join(__dirname, "./core-files/buildQuery.js"),
  path.join(__dirname, "./core-files/buildQuery.test.js"),
  path.join(__dirname, "./core-files/appendFilter.js"),
  path.join(__dirname, "./core-files/appendFilter.test.js")
];

const resolverAssetPaths = [
  path.join(__dirname, "./core-files/getResultsCursor.js"),
  path.join(__dirname, "./core-files/appendFilter.js")
];

const dataLibTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/DataLib.njk"),
  "utf8"
);

const initEntitiesTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/initEntities.njk"),
  "utf8"
);

const entityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/Entity.njk"),
  "utf8"
);

const createNestedEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/createNestedEntity.njk"),
  "utf8"
);

const updateNestedEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/updateNestedEntity.njk"),
  "utf8"
);

const deleteNestedEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/deleteNestedEntity.njk"),
  "utf8"
);

const queryEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/queryEntity.njk"),
  "utf8"
);

const getEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/getEntity.njk"),
  "utf8"
);

const countEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/countEntity.njk"),
  "utf8"
);

const createEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/createEntity.njk"),
  "utf8"
);

const updateEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/updateEntity.njk"),
  "utf8"
);

const updateEntityQueryTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/updateEntityQuery.njk"),
  "utf8"
);

const deleteEntityTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/deleteEntity.njk"),
  "utf8"
);

const deleteEntityQueryTemplate = fse.readFileSync(
  path.join(__dirname, "./templates/deleteEntityQuery.njk"),
  "utf8"
);

module.exports = function generateDataLib(tableMap, appConfig, opts) {
  opts = opts || {};
  let log = opts.log || function() {};

  let outputFiles = [];

  let njkEnv = new njk.Environment(null, {
    trimBlocks: true,
    lstripBlocks: true
  });

  //njkEnv.addFilter("applyNamingConvention", applyNamingConvention);

  let content;

  let njkResolverMethods = njk.compile(resolverMethodsTemplate, njkEnv);
  content = njkResolverMethods.render({ tableMap, makePlural, nameFormats });

  outputFiles.push({
    fullPath: path.join(appConfig.graphql.resolversPath, "index.js"),
    content: content
  });

  //let njkEntityMethods = njk.compile(entityMethodsTemplate, njkEnv);
  let njkDataLib = njk.compile(dataLibTemplate, njkEnv);
  content = njkDataLib.render({ tableMap, makePlural, nameFormats });

  outputFiles.push({
    fullPath: path.join(appConfig.graphql.dataLibPath, "index.js"),
    content: content
  });

  let njkInitEntities = njk.compile(initEntitiesTemplate, njkEnv);
  content = njkInitEntities.render({ tableMap, makePlural, nameFormats });

  outputFiles.push({
    fullPath: path.join(appConfig.graphql.dataLibPath, "entities/index.js"),
    content: content
  });

  let njkEntity = njk.compile(entityTemplate, njkEnv);
  let njkCreateNestedEntity = njk.compile(createNestedEntityTemplate, njkEnv);
  let njkUpdateNestedEntity = njk.compile(updateNestedEntityTemplate, njkEnv);
  let njkDeleteNestedEntity = njk.compile(deleteNestedEntityTemplate, njkEnv);
  let njkQueryEntity = njk.compile(queryEntityTemplate, njkEnv);
  let njkGetEntity = njk.compile(getEntityTemplate, njkEnv);
  let njkCreateEntity = njk.compile(createEntityTemplate, njkEnv);
  let njkCountEntity = njk.compile(countEntityTemplate, njkEnv);
  let njkUpdateEntity = njk.compile(updateEntityTemplate, njkEnv);
  let njkUpdateEntityQuery = njk.compile(updateEntityQueryTemplate, njkEnv);
  let njkDeleteEntity = njk.compile(deleteEntityTemplate, njkEnv);
  let njkDeleteEntityQuery = njk.compile(deleteEntityQueryTemplate, njkEnv);

  _.each(tableMap, (table, entityName) => {
    if (table.type === "view") {
      content = njkEntity.render({
        tableMap,
        table,
        entityName,
        makePlural,
        nameFormats,
        isView: true
      });
      outputFiles.push({
        fullPath: path.join(
          appConfig.graphql.dataLibPath,
          `entities/${table.modelName}/index.js`
        ),
        content: content
      });

      content = njkQueryEntity.render({
        tableMap,
        table,
        entityName,
        makePlural,
        nameFormats,
        hasJsonType
      });
      outputFiles.push({
        fullPath: path.join(
          appConfig.graphql.dataLibPath,
          `entities/${table.modelName}/query${entityName}.js`
        ),
        content: content
      });

      content = njkCountEntity.render({
        tableMap,
        table,
        entityName,
        makePlural,
        nameFormats
      });
      outputFiles.push({
        fullPath: path.join(
          appConfig.graphql.dataLibPath,
          `entities/${table.modelName}/count${entityName}.js`
        ),
        content: content
      });

      return;
    }
    content = njkEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/index.js`
      ),
      content: content
    });

    content = njkCreateNestedEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/createNested${entityName}Records.js`
      ),
      content: content
    });

    content = njkUpdateNestedEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/updateNested${entityName}Records.js`
      ),
      content: content
    });

    content = njkDeleteNestedEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/deleteNested${entityName}Records.js`
      ),
      content: content
    });

    content = njkQueryEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats,
      hasJsonType
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/query${entityName}.js`
      ),
      content: content
    });

    content = njkGetEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats,
      hasJsonType
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/get${entityName}.js`
      ),
      content: content
    });

    content = njkCountEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/count${entityName}.js`
      ),
      content: content
    });

    content = njkCreateEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/create${makePlural(entityName)}.js`
      ),
      content: content
    });

    content = njkUpdateEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/update${makePlural(entityName)}.js`
      ),
      content: content
    });

    content = njkUpdateEntityQuery.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/update${makePlural(entityName)}Query.js`
      ),
      content: content
    });

    content = njkDeleteEntity.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/delete${makePlural(entityName)}.js`
      ),
      content: content
    });

    content = njkDeleteEntityQuery.render({
      tableMap,
      table,
      entityName,
      makePlural,
      nameFormats
    });
    outputFiles.push({
      fullPath: path.join(
        appConfig.graphql.dataLibPath,
        `entities/${table.modelName}/delete${makePlural(entityName)}Query.js`
      ),
      content: content
    });
  });

  entityAssetPaths.forEach(assetPath => {
    if (fse.existsSync(assetPath)) {
      let content = fse.readFileSync(assetPath, "utf8");
      let assetName = path.basename(assetPath);
      let fullPath = path.join(
        appConfig.graphql.dataLibPath,
        `core/${assetName}`
      );
      outputFiles.push({
        fullPath,
        content
      });
    } else {
      console.warn(`Entity Asset ${assetPath} does not exist`);
    }
  });

  resolverAssetPaths.forEach(assetPath => {
    if (fse.existsSync(assetPath)) {
      let content = fse.readFileSync(assetPath, "utf8");
      let assetName = path.basename(assetPath);
      let fullPath = path.join(
        appConfig.graphql.resolversPath,
        `core/${assetName}`
      );
      outputFiles.push({
        fullPath,
        content
      });
    } else {
      console.warn(`Resolver Asset ${assetPath} does not exist`);
    }
  });

  let testFiles = generateDataLibTests(tableMap, appConfig, opts);

  outputFiles = [...outputFiles, ...testFiles];

  return outputFiles;
};

function makePlural(name) {
  return pluralize.plural(name);
}

function hasJsonType(table) {
  let result = false;

  _.each(table.attributes, attrDef => {
    if (attrDef.dataType === "ORDDOC") {
      result = true;
    }
  });

  return result;
}
