const { each, get } = require("lodash");

function addIndexes(tableMap, opts) {
  opts = opts || {};

  const { indexes: { excludedForeignKeys = [] } = {} } = opts;

  each(tableMap, (tableDef, entityName) => {
    if (tableDef.type !== "view") {
      const { tableName } = tableDef;

      let tableIndexes = get(tableDef, `indexes`, []);
      each(tableDef.attributes, (attrDef, attrName) => {
        if (attrDef.isForeignKey) {
          let tableQualifiedKeyName = `${tableName}.${attrDef.name}`;
          if (excludedForeignKeys.indexOf(tableQualifiedKeyName) < 0) {
            tableIndexes.push({
              sparseIndex: true,
              attributes: [attrName], // this get resolved in applyNamingRules
            });
          } else {
            console.warn(
              `Excluding foreign key: ${tableQualifiedKeyName} for auto indexing.`
            );
          }
        }
      });
      tableDef.indexes = tableIndexes;
    }
  });

  return tableMap;
}

module.exports = { addIndexes };
