const _ = require("lodash");

module.exports = function addTimestampColumns(tableMap, timestamps, opts) {
  opts = opts || {};

  _.each(tableMap, (table, tableName) => {
    if (table.type !== "view") {
      if (timestamps.created) {
        var createdColName = timestamps.created;
        if (typeof table.attributes[createdColName] === "undefined") {
          table.attributes[createdColName] = {
            name: createdColName,
            dataType: "TIMESTAMP",
            isPrimaryKey: false,
            isForeignKey: false,
            size: "",
            precision: "",
            scale: "",
            nullable: true,
            unique: false,
            columnName: createdColName,
            timestamp: "created",
          };
        } else {
          if (timestamps.errorOnConflict) {
            throw new Error(
              `[DM GEN ERROR] timestamp ${createdColName} field already exists on ${tableName}. Unset timestamps.errorOnConflict to warn instead of throwing.`
            );
          } else {
            console.warn(
              `[DM GEN WARNING] timestamp ${createdColName} field already exists on ${tableName}. Set timestamps.errorOnConflict throw an error instead of warning`
            );
          }
        }
      }

      if (timestamps.modified) {
        var modifiedColName = timestamps.modified;
        if (typeof table.attributes[modifiedColName] === "undefined") {
          table.attributes[modifiedColName] = {
            name: modifiedColName,
            dataType: "TIMESTAMP",
            isPrimaryKey: false,
            isForeignKey: false,
            size: "",
            precision: "",
            scale: "",
            nullable: true,
            unique: false,
            columnName: modifiedColName,
            timestamp: "modified",
          };
        } else {
          if (timestamps.errorOnConflict) {
            throw new Error(
              `[DM GEN ERROR] timestamp ${modifiedColName} field already exists on ${tableName}. Unset timestamps.errorOnConflict to warn instead of throwing.`
            );
          } else {
            console.warn(
              `[DM GEN WARNING] timestamp ${modifiedColName} field already exists on ${tableName}. Set timestamps.errorOnConflict throw an error instead of warning`
            );
          }
        }
      }
    }
  });

  return tableMap;
};
