const _ = require("lodash");
const clone = require("../graphql-generator/clone");

const nidTemplate = {
  name: "nid",
  dataType: "NUMERIC",
  isPrimaryKey: false,
  isForeignKey: false,
  isReadOnly: true,
  size: "",
  precision: "",
  scale: "",
  nullable: true,
  defaultValue: "",
  autoIncrement: true,
  columnName: "nid",
};

module.exports = function extendTableMap(tableMap, opts) {
  opts = opts || {};
  var tableMapExtender = opts.extendTableMap;

  addCidFields(tableMap, opts);

  if (tableMapExtender) {
    tableMap = tableMapExtender(tableMap, opts);
  }

  if (opts.useUUIDs) {
    tableMap = convertNumericIdsToUUIDs(tableMap, opts);
  }

  return tableMap;
};

function addCidFields(tableMap, opts) {
  _.each(tableMap, (table, tableName) => {
    if (typeof table.attributes["cid"] === "undefined") {
      table.attributes["cid"] = {
        name: "cid",
        dataType: "VARCHAR",
        isPrimaryKey: false,
        isForeignKey: false,
        size: "500",
        precision: "",
        scale: "",
        nullable: true,
        unique: true,
        columnName: "cid",
      };
    } else {
      throw new Error(
        "[DM GEN ERROR] cid field already exists on " + tableName
      );
    }
  });
}
function convertNumericIdsToUUIDs(tableMap, opts) {
  _.each(tableMap, (table, tableName) => {
    let hasNumericId = false;
    _.each(table.attributes, (attr, attrName) => {
      if (
        (attr.isPrimaryKey || attr.isForeignKey) &&
        attr.dataType === "NUMERIC"
      ) {
        if (attr.isPrimaryKey) {
          hasNumericId = true;
          attr.defaultRawValue = "gen_random_uuid()";
          delete attr.defaultValue;
          delete attr.autoIncrement;
        }
        attr.dataType = "UUID";
      }
    });
    if (table.attributes) {
      table.attributes["nid"] = clone(nidTemplate);
      if (hasNumericId) {
        table.attributes["legacy_id"] = {
          name: "legacy_id",
          dataType: "NUMERIC",
          isPrimaryKey: false,
          isForeignKey: false,
          size: "",
          precision: "",
          scale: "",
          nullable: true,
          defaultValue: "",
          columnName: "legacy_id",
        };
      }
    }
  });
  return tableMap;
}
