const parseOraCSV = require("./parseOraCSV");
const convertRowsToTableMap = require("./convertRowsToTableMap");
const applyNamingRules = require("./applyNamingRules");
const populateRelationshipAttributes = require("./populateRelationshipAttributes");
const extendTableMap = require("./extendTableMap");
const addTimestampColumns = require("./addTimestampColumns");
const addUpdatedByColumn = require("./addUpdatedByColumn");
const { addIndexes } = require("./addIndexes");
const { processViews } = require("./views");
const { mergeCommonTableMap } = require("./mergeCommonTableMap");
// const { defaultBooleans } = require("./defaultBooleans");

module.exports = function parseDatamodelCSV(
  csvPath,
  commonSchemaCsvPath,
  optsIn
) {
  let opts = optsIn || {};
  let log = opts.log || function () {};

  let csvData = parseOraCSV(csvPath, opts);
  let tableMap = convertRowsToTableMap(
    csvData.attributes,
    csvData.entities,
    csvData.relations,
    csvData.keyAttributes,
    csvData.comments,
    opts
  );

  if (commonSchemaCsvPath) {
    let commonCSVData = parseOraCSV(commonSchemaCsvPath, opts);
    let commonTableMap = convertRowsToTableMap(
      commonCSVData.attributes,
      commonCSVData.entities,
      commonCSVData.relations,
      commonCSVData.keyAttributes,
      commonCSVData.comments,
      opts
    );
    tableMap = mergeCommonTableMap(commonTableMap, tableMap, opts);
  }

  tableMap = extendTableMap(tableMap, opts);

  if (opts.timestamps) {
    tableMap = addTimestampColumns(tableMap, opts.timestamps, opts);
  }

  if (opts.updatedBy) {
    tableMap = addUpdatedByColumn(tableMap, opts.updatedBy, opts);
  }

  tableMap = addIndexes(tableMap, opts);
  // tableMap = defaultBooleans(tableMap, opts);
  tableMap = applyNamingRules(tableMap, opts);
  tableMap = processViews(tableMap, opts);
  tableMap = populateRelationshipAttributes(tableMap, opts);

  return tableMap;
};
