const { each, cloneDeep } = require("lodash");

function mergeCommonTableMap(commonTableMap, tableMap, opts) {
  const overrideWithModelFromCommon =
    (opts.extendTableMap && opts.extendTableMap.overrideWithModelFromCommon) ||
    [];

  each(commonTableMap, (entityDef, entityName) => {
    // overwrite existing user table, so we can leave that in the datamodel as a placeholder
    if (tableMap[entityName]) {
      if (
        entityName === "User" ||
        overrideWithModelFromCommon.includes(entityName)
      ) {
        console.warn(
          `Overwriting ${entityName} table definition with model from common`
        );
      } else {
        throw new Error(
          `Common Schema Entity ${entityName} already exists in app schema!`
        );
      }
    }

    tableMap[entityName] = cloneDeep(entityDef);
    tableMap[entityName].isCommonEntity = true;
  });
  return tableMap;
}

module.exports = { mergeCommonTableMap };
