const csvParser = require("babyparse");
const path = require("path");
const fse = require("fs-extra");

module.exports = function(csvFolder, opts) {
  opts = opts || {};
  var log = opts.log || function() {};

  log("Convert Oracle Data Modeler Logical CSV's to JSON");

  var entitiesPath = path.join(csvFolder, "DM_Entities.csv");
  var attributesPath = path.join(csvFolder, "DM_Attributes.csv");
  var relationsPath = path.join(csvFolder, "DM_Relations.csv");
  var keyAttributesPath = path.join(csvFolder, "DM_Key_Attributes.csv");
  var commentsPath = path.join(csvFolder, "DM_Comments.csv");

  var attributes = parseCsvFile(attributesPath).data;
  var entities = parseCsvFile(entitiesPath).data;
  var relations = parseCsvFile(relationsPath).data;
  var keyAttributes = parseCsvFile(keyAttributesPath).data;
  var comments = parseCsvFile(commentsPath).data;

  return {
    attributes: attributes,
    entities: entities,
    relations: relations,
    keyAttributes: keyAttributes,
    comments: comments
  };

  function parseCsvFile(csvPath, verbose) {
    if (Array.isArray(csvPath)) {
      var results = [];
      csvPath.forEach(function(item) {
        var result = parseCsvFile(item);
        results.push(result);
      });
      return results;
    } else {
      log("Parsing: " + csvPath);
      if (!fse.existsSync(csvPath)) {
        throw new Error(`Missing data modeler CSV export ${csvPath}`);
      }
      var parsedResults = csvParser.parseFiles(csvPath, { header: true });
      log("Parsing complete");
      log(parsedResults);
      return parsedResults;
    }
  }
};
