const _ = require('lodash');
const pluralize = require('pluralize');

module.exports = function populateRelationshipAttributes(tableMap, opts){
    opts = opts || {};
    var log = opts.log || function () { };

    _.each(tableMap, (table, tableName) => {
        var modelAttributes = table.modelAttributes;

        var associationNames = {};

        // _.each(table.attributes, (attrDef, attrName) => {
        //     modelAttributes[attrName] = true;
        // })

        _.each(table.attributes, (attrDef, name) => {
            var attrName = attrDef.name;
            if(attrDef.isForeignKey){
                var relAttrName = "";

                if(_.endsWith(name, "_id")){
                    relAttrName = name.substr(0, name.length - 3);
                    relAttrName =  _.camelCase(relAttrName);
                }else if(_.endsWith(name, "_code")){
                    relAttrName = name.substr(0, name.length - 4);
                    relAttrName =  _.camelCase(relAttrName);
                }

                if(modelAttributes[relAttrName] || relAttrName == ""){
                    relAttrName = tableMap[attrDef.referenceName].modelName;
                }
                
                if(modelAttributes[relAttrName]){
                    relAttrName = attrName + "_" + tableMap[attrDef.referenceName].modelName;
                }

                attrDef.relAttrName = relAttrName;
                modelAttributes[relAttrName] = true;
            }
        });

        _.each(table.associations, (assoc) => {

            if(assoc.referenceKey.lastIndexOf('_') > -1){
                relAttrName = assoc.name + assoc.referenceKey.substr(0, assoc.referenceKey.lastIndexOf('_'));
            }else{
                relAttrName = assoc.name + assoc.referenceKey;
            }

            if(assoc.isPlural){
                relAttrName = pluralize.plural(relAttrName);
            }

            // if(modelAttributes[relAttrName] || 
            //     (relCursorAttrName != "" && modelAttributes[relCursorAttrName])){
            //     relAttrName = assoc.name + "_" + attrDef.referenceKey;
            //     relCursorAttrName = relAttrName + "Cursor";
            // }

            var shortName = assoc.name;

            associationNames[shortName] = associationNames[shortName] || [];
            associationNames[shortName].push({
                longName: relAttrName,
                referenceKey: assoc.referenceKey
            });
        });

        //log("Association Names: " + tableName);
        //log(associationNames);

        _.each(table.associations, (assoc) => {
            var shortName = assoc.name;            

            if(associationNames[shortName].length === 1){
                    modelAttributes[shortName] = true;
                    if(assoc.isPlural)
                        modelAttributes[shortName + "Cursor"] = true;
            }else{
                var assocName = associationNames[shortName].find((item) => item.referenceKey === assoc.referenceKey);
                assoc.name = assocName.longName;
                modelAttributes[assoc.name] = true;
                if(assoc.isPlural)
                    modelAttributes[assoc.name + "Cursor"] = true;

            }
        });

        table.modelAttributes = modelAttributes;
    });

    return tableMap;
}
