const _ = require("lodash");

module.exports = function extendTableMap(tableMap, opts) {
  _.each(tableMap, (table, tableName) => {
    if (typeof table.attributes["createdAt"] === "undefined") {
      table.attributes["createdAt"] = {
        name: "createdAt",
        dataType: "Timestamp",
        isPrimaryKey: false,
        isForeignKey: false,
        size: "",
        precision: "",
        scale: "",
        nullable: true,
        columnName: "createdAt"
      };
    }
  });

  tableMap["BodyPreview"] = {
    type: "view",
    columns: {
      postId: {
        relatedKeyPath: "Post.id",
        oneToOne: true,
        relAttrName: "post" //optional
      },
      bodyPreview: {
        dataType: "TEXT"
      },
      title: {
        relatedAttributePath: "Post.title"
      }
    },
    primaryKeyField: "postId",
    sql: {
      pg: `
            SELECT 
                {{{post.id}}} as "postId",
                {{{post.title}}} as "title",
                substr({{{post.body}}}, 0, 95) ||
                CASE WHEN length({{{post.body}}}) > 95 
                    THEN '...'
                    ELSE ''
                END as "bodyPreview"
            FROM
                {{{tables.post}}}
        `
    }
  };

  return tableMap;
};
