const { each, camelCase } = require("lodash");
const { getViews, setViews } = require("./getViews");
const log = require("../../utils/log");
const { getAttributeForViewColumn } = require("./getAttributeForViewColumn");
function populateAttributes(tableMap) {
  const views = getViews(tableMap);

  // log("Views to format", views);

  views.forEach(view => {
    let attributes = {};
    let modelAttributes = {};
    let columnNames = {};

    let tableName = camelCase(view.viewName);
    view.tableName = tableName;
    view.modelName = tableName;

    each(view.columns, (val, name) => {
      let attr = getAttributeForViewColumn(
        view.viewName,
        tableName,
        { ...val, name },
        tableMap
      );
      attributes[attr.name] = attr;
      modelAttributes[attr.name] = true;
      if(attr.relAttrName){
        if(!modelAttributes[attr.relAttrName]){
          modelAttributes[attr.relAttrName] = true;
        } else {
          throw new Error(`Related Entity Name relAttrName: ${attr.relAttrName} already exists of model for view ${viewName}, column ${attr.name}`)
        }
      }
      // don't forget relAttrName
      columnNames[attr.columnName] = true;
    });

    view.attributes = attributes;
    view.modelAttributes = modelAttributes;
    view.columnNames = columnNames;
    view.associations = []; //views currently don't support FK's so they will never have associations

    //normalize the view columns
    //loop through each column
    // + convert column to attribute
    // + add column to columnNames
    // + add attributes to modelAttributes
    //set modelName
  });
  setViews(views, tableMap);
  return tableMap;
}

module.exports = {
  populateAttributes
};
