const _ = require('lodash');

module.exports = function (schemaDef, opts) {
    var schemaStr = "";

    _.each(schemaDef.scalars, (scalarName) => {
        schemaStr += " scalar " + scalarName + "\n";
    });

    schemaStr += "\n";

    _.each(schemaDef.types, (typeDef, typeName) => {
        if(typeDef._comment){
            schemaStr += "# " + typeDef._comment + " \n";
        }
        schemaStr += " type " + typeName + " {\n";
        _.each(typeDef, (attrDef, attrName) => {
            if(attrName === "_comment") return;
            
            if (attrDef.comment) {
                schemaStr += " # " + attrDef.comment + " \n";
            }
            var args = "";
            if (attrDef.args) {
                args = "(";
                _.each(attrDef.args, (argDef, argName) => {
                    if (args.length > 1) args += ", ";

                    if (argDef.isList) {
                        args += argName + ": [" + argDef.type + "]";
                    } else {
                        args += argName + ": " + argDef.type;
                        if (argDef.required) args += "!";
                    }
                });
                args += ")";
            }
            if (attrDef.isList) {
                schemaStr += "  " + attrName + args + ": [" + attrDef.type + "]\n";
            } else {
                schemaStr += "  " + attrName + args + ": " + attrDef.type;
                if (attrDef.required) schemaStr += "!";
                schemaStr += "\n";
            }
        });
        schemaStr += " }\n\n";
    });

    _.each(schemaDef.inputs, (typeDef, typeName) => {
        schemaStr += " input " + typeName + " {\n";
        _.each(typeDef, (attrDef, attrName) => {
            if (attrDef.isList) {
                schemaStr += "  " + attrName + ": [" + attrDef.type + "]\n";
            } else {
                schemaStr += "  " + attrName + ": " + attrDef.type;
                if (attrDef.required) schemaStr += "!";
                schemaStr += "\n";
            }
        });
        schemaStr += " }\n\n";
    });


    schemaStr += "\n";
    schemaStr += " type Query {\n";

    _.each(schemaDef.queries, (attrDef, attrName) => {
        var args = "";
        if (attrDef.args) {
            args = "(";
            _.each(attrDef.args, (argDef, argName) => {
                if (args.length > 1) args += ", ";

                if (argDef.isList) {
                    args += argName + ": [" + argDef.type + "]";
                } else {
                    args += argName + ": " + argDef.type;
                    if (argDef.required) args += "!";
                }
            });
            args += ")";
        }
        if (attrDef.isList) {
            schemaStr += "  " + attrName + args + ": [" + attrDef.type + "]\n";
        } else {
            schemaStr += "  " + attrName + args + ": " + attrDef.type + "\n";
        }
    });
    schemaStr += " }\n";

    if (_.keys(schemaDef.mutations).length > 0) {
        schemaStr += "\n";
        schemaStr += " type Mutation {\n";

        _.each(schemaDef.mutations, (attrDef, attrName) => {
            var args = "";
            if (attrDef.args) {
                args = "(";
                _.each(attrDef.args, (argDef, argName) => {
                    if (args.length > 1) args += ", ";

                    if (argDef.isList) {
                        args += argName + ": [" + argDef.type + "]";
                    } else {
                        args += argName + ": " + argDef.type;
                        if (argDef.required) args += "!";
                    }
                });
                args += ")";
            }

            if (attrDef.isList) {
                schemaStr += "  " + attrName + args + ": [" + attrDef.type + "]\n";
            } else {
                schemaStr += "  " + attrName + args + ": " + attrDef.type + "\n";
            }
        });
        schemaStr += " }\n";
    }

    return schemaStr;

}