const _ = require("lodash");
const extractQueryResultTypes = require("./extractQueryResultTypes");
const extractCreateMutationTypes = require("./extractCreateMutationTypes");
const extractUpdateMutationTypes = require("./extractUpdateMutationTypes");
const extractDeleteMutationTypes = require("./extractDeleteMutationTypes");
const convertToGraphQLType = require("./convertToGraphQLType");
const { formatQueryResultName } = require("./name-formats");
module.exports = function convertTableToTypeDef(
  tableMap,
  tableDef,
  modelName,
  schemaDef
) {
  var attrStrDef = "";
  // var typeAttrs = {
  //     "_resultInfo": { type: 'JSON', isScalar: true }
  // };
  var typeAttrs = {};

  _.each(tableDef.attributes, (attrDef, key) => {
    if (attrDef.isForeignKey && !tableMap[attrDef.referenceName]) {
      // if the attribute is a foreign and the reference doesn't exist we assume it's a cross schema common reference
      return;
    }
    var attrName = attrDef.name;
    typeAttrs[attrName] = {
      type: convertToGraphQLType(attrDef, attrName),
      required: !attrDef.nullable,
      isScalar: true,
      comment: attrDef.comment,
      isKey: attrDef.isPrimaryKey,
    };

    if (attrDef.isReadOnly) {
      typeAttrs[attrName].isReadOnly = true;
    }

    if (attrDef.isForeignKey || attrDef.isRelatedKey) {
      typeAttrs[attrName].required = false;

      var attrRefName = attrDef.relAttrName;

      typeAttrs[attrName].comment = `Reference key for ${attrRefName}`;

      typeAttrs[attrRefName] = {
        type: tableMap[attrDef.referenceName].modelName,
        required: false,
        comment: attrDef.comment,
      };
    }
  });

  typeAttrs.lastFetched = {
    type: "Date",
    required: false,
    isScalar: true,
    comment:
      "Server generated property that indicates the last time this attribute was fetched",
  };

  _.each(tableDef.associations, (assoc) => {
    if (!tableMap[assoc.referenceName]) {
      // if the association doesn't exist we assume it's a cross schema common reference
      return;
    }

    typeAttrs[assoc.name] = {
      type: tableMap[assoc.referenceName].modelName,
      required: false,
      isList: assoc.isPlural,
      comment: assoc.comment,
      isReadOnly: assoc.isReadOnly,
      args: {
        filter: { type: "JSON" },
        sort: { type: "[String]" },
        pageNumber: { type: "Int" },
        pageSize: { type: "Int" },
      },
    };

    typeAttrs[assoc.name + "Cursor"] = {
      type: formatQueryResultName(tableMap[assoc.referenceName].modelName),
      required: false,
      isCursor: true,
      comment: `Returns a paginated cursor result of ${assoc.name} instead of an Array`,
      isReadOnly: assoc.isReadOnly,
      args: {
        filter: { type: "JSON" },
        sort: { type: "[String]" },
        pageNumber: { type: "Int" },
        pageSize: { type: "Int" },
      },
    };
  });

  schemaDef.types[modelName] = typeAttrs;
  extractQueryResultTypes(modelName, typeAttrs, schemaDef);
  if (tableDef.type !== "view") {
    extractCreateMutationTypes(modelName, typeAttrs, schemaDef);
    extractUpdateMutationTypes(modelName, typeAttrs, schemaDef);
    extractDeleteMutationTypes(modelName, typeAttrs, schemaDef);
  }

  _.each(typeAttrs, (val) => {
    delete val.isReadOnly;
  });

  if (tableDef.comment && tableDef.comment != "") {
    typeAttrs._comment = tableDef.comment;
  }
};
