const _ = require('lodash');
const clone = require('./clone');
const { formatCreateInputName,
        formatBulkCreateInputName, 
        formatCreatePayloadName,
        formatBulkCreatePayloadName,
        formatQueryResultName } = require('./name-formats');

module.exports = function extractCreateMutationTypes(modelName, typeAttrs, schemaDef){
    var keysToDelete = [];
    var input = clone(typeAttrs);

    _.each(input, (attrDef, attrName) => {

        if(attrDef.isCursor || attrDef.isReadOnly){
            keysToDelete.push(attrName);
            return;
        }

        if(!attrDef.isScalar){
            attrDef.type = formatCreateInputName(attrDef.type);
        }
        if(attrDef.isKey) attrDef.required = false;
    });

    keysToDelete.forEach((keyName) => {
        delete input[keyName];
    });

    schemaDef.inputs[formatCreateInputName(modelName)] = input;
    schemaDef.inputs[formatBulkCreateInputName(modelName)] = {
        createItems: {
            type: formatCreateInputName(modelName),
            isList: true
        }
    };
    schemaDef.types[formatCreatePayloadName(modelName)] = {
        createdItemsCursor: {
            type: formatQueryResultName(modelName)
        }
    };
}
