const _ = require("lodash");
const clone = require("./clone");
const {
  formatDeleteInputValuesName,
  formatDeleteInputName,
  formatBatchDeleteInputName,
  formatDeletePayloadName,
  formatBatchDeletePayloadName
} = require("./name-formats");

module.exports = function extractDeleteMutationTypes(
  modelName,
  typeAttrs,
  schemaDef
) {
  var input = {};

  _.each(typeAttrs, (attrDef, attrName) => {
    if (attrDef.isCursor || attrDef.isReadOnly) return;
    if (attrDef.isKey) {
      input[attrName] = clone(attrDef);
    } else if (!attrDef.isScalar) {
      input[attrName] = clone(attrDef);
      input[attrName].type = formatDeleteInputName(attrDef.type);
      input[attrName].required = false;
    }
  });

  schemaDef.inputs[formatDeleteInputName(modelName)] = input;

  schemaDef.inputs[formatBatchDeleteInputName(modelName)] = {
    deleteQueries: {
      type: "JSON",
      isScalar: true,
      isList: true
    }
  };

  schemaDef.types[formatDeletePayloadName(modelName)] = {
    deletedCount: {
      type: "Int",
      isScalar: true
    }
  };

  schemaDef.types[formatBatchDeletePayloadName(modelName)] = {
    deletedCount: {
      type: "Int",
      isScalar: true
    }
  };
};
