const _ = require("lodash");
const pluralize = require("pluralize");
const convertTableToTypeDef = require("./convertTableToTypeDef");
const convertToGraphQLType = require("./convertToGraphQLType");
const {
  formatQueryName,
  formatQueryResultName,
  formatCreateMutationName,
  formatCreateInputName,
  formatCreatePayloadName,
  formatBulkCreateMutationName,
  formatBulkCreateInputName,
  formatBulkCreatePayloadName,
  formatUpdateMutationName,
  formatUpdateInputName,
  formatUpdatePayloadName,
  formatBulkUpdateMutationName,
  formatBulkUpdateInputName,
  formatBulkUpdatePayloadName,
  formatBatchUpdateMutationName,
  formatBatchUpdateInputName,
  formatUpdateQueryInputName,
  formatBatchUpdatePayloadName,
  formatDeleteMutationName,
  formatDeleteInputName,
  formatDeletePayloadName,
  formatBulkDeleteMutationName,
  formatBulkDeleteInputName,
  formatBulkDeletePayloadName,
  formatBatchDeleteMutationName,
  formatBatchDeleteInputName,
  formatBatchDeletePayloadName
} = require("./name-formats");

module.exports = function(modelSpec, opts) {
  //var typeDefStr = "const typeDefs = `\n\n";
  var schemaDef = {
    scalars: ["JSON", "Date", "Number"],
    types: {},
    inputs: {},
    queries: {},
    mutations: {}
  };

  _.each(modelSpec, (tableDef, entityName) => {
    var modelName = tableDef.modelName;

    convertTableToTypeDef(modelSpec, tableDef, modelName, schemaDef);

    schemaDef.queries[pluralize.plural(modelName)] = {
      type: formatQueryResultName(modelName),
      isList: false,
      args: {
        filter: { type: "JSON" },
        sort: { type: "[String]" },
        pageNumber: { type: "Int" },
        pageSize: { type: "Int" }
      }
    };

    if (tableDef.type !== "view") {
      //Query
      schemaDef.queries[modelName] = {
        type: modelName,
        isList: false,
        args: {
          [tableDef.primaryKeyField]: {
            type: convertToGraphQLType(
              tableDef.attributes[tableDef.primaryKeyField],
              tableDef.primaryKeyField
            ),
            required: true,
            isScalar: true
          }
        }
      };

      //Create
      schemaDef.mutations[formatCreateMutationName(modelName)] = {
        type: formatCreatePayloadName(modelName),
        args: {
          input: {
            type: formatCreateInputName(modelName),
            isList: true,
            required: true
          }
        }
      };

      //Update
      schemaDef.mutations[formatUpdateMutationName(modelName)] = {
        type: formatUpdatePayloadName(modelName),
        args: {
          input: {
            type: formatUpdateInputName(modelName),
            isList: true,
            required: true
          }
        }
      };

      schemaDef.mutations[formatBatchUpdateMutationName(modelName)] = {
        type: formatBatchUpdatePayloadName(modelName),
        args: {
          input: {
            type: formatBatchUpdateInputName(modelName),
            required: true
          }
        }
      };

      //Delete
      schemaDef.mutations[formatDeleteMutationName(modelName)] = {
        type: formatDeletePayloadName(modelName),
        args: {
          input: {
            type: formatDeleteInputName(modelName),
            isList: true,
            required: true
          }
        }
      };

      schemaDef.mutations[formatBatchDeleteMutationName(modelName)] = {
        type: formatBatchDeletePayloadName(modelName),
        args: {
          input: {
            type: formatBatchDeleteInputName(modelName),
            required: true
          }
        }
      };
    }
  });

  return schemaDef;
};
