const typeMap = {
  VARCHAR: convertVarchar,
  NUMERIC: convertNumeric,
  NVARCHAR: convertVarchar,
  ORDDOC: convertOradoc,
  TINYINT: convertInteger,
  TEXT: convertText,
  TIMESTAMP: convertTimestamp,
  INTEGER: convertInteger,
  BOOLEAN: convertBoolean,
  CLOB: convertVarchar,
  UUID: convertUUID,
};

function convertVarchar(attrDef, attrName, dialect) {
  var size = Number(attrDef.size || 400);
  return {
    type: "string",
    typeArgs: [attrName, size],
  };
}

function convertNumeric(attrDef, attrName, dialect) {
  if (dialect === "postgres") {
    if (attrDef.isForeignKey) return convertInteger(attrDef, attrName, dialect);

    return {
      type: "specificType",
      typeArgs: [attrName, "NUMERIC"],
    };
  }
  if (dialect === "oracle") {
    return {
      type: "specificType",
      typeArgs: [attrName, "NUMBER"],
    };
  }
  return {
    type: "decimal",
    typeArgs: [attrName],
  };
}
function convertOradoc(attrDef, attrName, dialect) {
  if (dialect === "oracle") {
    return {
      type: "specificType",
      typeArgs: [attrName, "CLOB"],
    };
  }
  return {
    type: "json",
    typeArgs: [attrName],
  };
}
function convertText(attrDef, attrName, dialect) {
  if (dialect === "oracle") {
    return {
      type: "specificType",
      typeArgs: [attrName, "CLOB"],
    };
  }
  return {
    type: "text",
    typeArgs: [attrName],
  };
}

function convertTimestamp(attrDef, attrName, dialect) {
  return {
    type: "timestamp",
    typeArgs: [attrName],
  };
}

function convertInteger(attrDef, attrName, dialect) {
  return {
    type: "integer",
    typeArgs: [attrName],
  };
}

function convertBoolean(attrDef, attrName, dialect) {
  return {
    type: "boolean",
    typeArgs: [attrName],
  };
}

function convertUUID(attrDef, attrName, dialect) {
  return {
    type: "varchar",
    typeArgs: [attrName, 500],
  };
}

module.exports = { typeMap };
