const fse = require('fs-extra');
const path = require('path');
const _ = require('lodash');
const knex = require('knex');

const GraphQLNumber = require('../graphql-scalars/GraphQLNumber');
const GraphQLDate = require('graphql-date');
const GraphQLJSON = require('graphql-type-json');

module.exports = function loadResolvers({ DataLib, db }, appConfig, opts){

    var resolverPath = appConfig.graphql.resolversPath;
    var enableModelResolvers = true;

    if(appConfig.graphql.enableModelResolvers != null &&
       appConfig.graphql.enableModelResolvers == false){
        enableModelResolvers = false;
    }

    var resolvers = {};

    if(resolverPath && enableModelResolvers){

        var coreResolverPath = path.join(resolverPath, 'index.js');
        //tnr: commenting out the ext resolvers extension in favor of an inline resolver extension
        //check the bottom of resolverMethods.njk for the code
        
        if(fse.existsSync(coreResolverPath)){
            resolvers = require(coreResolverPath)(db, DataLib);
        }
    }

    resolvers = _.assign(resolvers, {
        JSON: GraphQLJSON,
        Date: GraphQLDate,
        Number: GraphQLNumber
    });

    return resolvers;
}