var assertErr = require('assert-err')
var GraphQLScalarType = require('graphql').GraphQLScalarType
var GraphQLError = require('graphql/error').GraphQLError
var Kind = require('graphql/language').Kind

module.exports = new GraphQLScalarType({
  name: 'Number',
  /**
   * Serialize number value into string
   * @param  {Date} value date value
   * @return {String} date as string
   */
  serialize: function (numValue) {
    var value = Number(numValue);
    assertErr((value instanceof Number || value.constructor === Number), TypeError, 'Field error: value is not an instance of Number ' )
    assertErr(!isNaN(value), TypeError, 'Field error: value is an invalid Number')
    return value + 0;
  },
  /**
   * Parse value into date
   * @param  {*} value serialized date value
   * @return {Date} date value
   */
  parseValue: function (value) {
    var num = new Number(value)
    assertErr(!isNaN(num), TypeError, 'Field error: value is an invalid Number')
    return num + 0;
  },
  /**
   * Parse ast literal to date
   * @param  {Object} ast graphql ast
   * @return {Date} date value
   */
  parseLiteral: function (ast) {
    assertErr(ast.kind === Kind.STRING || ast.kind === Kind.INT || ast.kind === Kind.FLOAT,
      GraphQLError, 'Query error: Can only parse strings, floats, and integers to number but got a: ' + ast.kind, [ast])

    var result = new Number(ast.value)
    assertErr(!isNaN(result),
      GraphQLError, 'Query error: Invalid number', [ast])
  
    return result + 0;
  }
})