
const cleanSchema = require('./cleanSchema');
const createTables = require('./createTables');
const createRelationships = require('./createRelationships');
const createConstraints = require('./createConstraints');
const createFunctions = require('./createFunctions');
const createTriggers = require('./createTriggers');
const runInitSql = require('./runInitSql');
const fs = require('fs');
module.exports = function(db, schemaSpec, opts, appConfig){

    let applyBindings = (data) => {
      if(!data) return;
      try{
        var bindingsArray = JSON.parse(JSON.stringify(data.bindings));
        var sql = JSON.parse(JSON.stringify(data.cmd));
      }
      catch(e){
        console.error("Cannot parse data")
        console.log(data)
        return ""
      }
        while(sql.indexOf("?")!==-1){
            var replacing = bindingsArray.shift();
            if(typeof replacing === "string") { replacing = '\''+replacing+'\''; }
            if(typeof replacing === "object") { replacing = JSON.stringify(replacing) }
            sql = sql.replace("?",replacing);
        }
        return sql
    }

    db.logSQL = (qb) => {
      if (!process.env.generateSQL) return qb;
      let data
      if(qb.cmd&&!qb.toSQL)
      {
        qb.cmd = qb.cmd.replace(/CREATE FUNCTION/g,'CREATE OR REPLACE FUNCTION');
        qb.cmd = qb.cmd.replace(/CREATE TRIGGER/g,'CREATE OR REPLACE TRIGGER');
        qb.cmd = qb.cmd.replace(/references\ /g,'references "app".');
        qb.cmd = qb.cmd.replace(/\n\$BODY\$/g,'\n$BODY$;');
        data = applyBindings(qb)
        if(!data.match(/BEGIN/)&&data.charAt(data.length-1)!==";") { data += ";" }
      }
      else if(qb.toString()&&qb.toSQL)
      {
        data = qb.toString()
        data = data.replace(/CREATE FUNCTION/g,'CREATE OR REPLACE FUNCTION');
        data = data.replace(/CREATE TRIGGER/g,'CREATE OR REPLACE TRIGGER');
        data = data.replace(/references\ /g,'references "app".');
        data = data.replace(/\n\$BODY\$/g,'\n$BODY$;');

        if(qb.toString&&qb.toSQL){
          if(!data.match(/BEGIN/)&&data.charAt(data.length-1)!==";") { data += ";" }
        }
      }
      else { data = "\n-- "+qb+"\n"; }
      fs.appendFileSync(process.env.generateSQLPath, data+"\n");
      return qb
    }

    if(process.env.generateSQL) {
      let generatedSQLPath = '/tmp/output.sql';
      if (process.env.generateSQLPath) generateSQLPath = process.env.generateSQLPath;
      else process.env.generateSQLPath = generatedSQLPath;
      console.log(`Generating output SQL at: ${generatedSQLPath}`);
      fs.writeFileSync(generatedSQLPath,"",{encoding:'utf8',flag:'w'})
    }

    return cleanSchema(db, schemaSpec.cleanSchema, opts)
           .then(function(){
                return createTables(db, schemaSpec.tables, opts);
           })
           .then(function(){
                return createRelationships(db, schemaSpec.relationships, opts);
           })
           .then(function(){
                return createConstraints(db, schemaSpec.constraints, opts);
           })
          .then(function(){
               return createFunctions(db, schemaSpec.functions, opts);
          })
          .then(function(){
               return createTriggers(db, schemaSpec.triggers, opts);
          })
          .then(function(){
               return runInitSql(db, schemaSpec.initSql, opts);
          })
          .then(function(){
            if(appConfig.db.dialect==="postgres"){
              // return db.raw(`CREATE DATABASE mydb_template TEMPLATE ${appConfig.db.dbName};`)
              // TODO: This was intended be created for using
              // http://blog.endpoint.com/2010/05/postgresql-template-databases-to.html
              // Getting the following error: There are 2 other sessions using the database
              return Promise.resolve();
            }
            else return Promise.resolve();
          })
        ;
}