const Promise = require("bluebird");

async function createConstraints(db, constraints, opts = {}) {
  const { logDebug = () => {}, moduleSchema, commonSchema } = opts;

  db.logSQL(`Creating constrains`);

  for (let con of constraints) {
    const {
      isCommon,
      schema,
      constraintType,
      tableName,
      type,
      typeArgs,
      name,
    } = con;

    const constraintName = `${name} - ${constraintType}`;
    logDebug(`[ORADM-CONSTRAINTS] Creating constraint ${constraintName}`);

    if (!isCommon && !moduleSchema) {
      logDebug(
        `[ORADM-CONSTRAINTS] Skipping constraint ${constraintName} because it's in the module schema`
      );
      continue;
    }

    if (isCommon && !commonSchema) {
      logDebug(
        `[ORADM-CONSTRAINTS] Skipping constraint ${constraintName} because it's in the common schema`
      );
      continue;
    }

    let qb = db.schema
      .withSchema(schema)
      .alterTable(tableName, function (table) {
        if (constraintType === "NOT_NULL") {
          var col = table[type].apply(this, typeArgs);
          col.notNullable().alter();
        } else {
          throw new Error("Constraint Type Not Recognized: " + constraintType);
        }
      });

    await db.logSQL(qb);

    logDebug(
      `[ORADM-CONSTRAINTS] Created constraint ${constraintName} on table ${tableName}`
    );
  }
}

module.exports = createConstraints;
