const Promise = require("bluebird");

function createRelationships(db, relationships, opts = {}) {
  const { logDebug = () => {}, commonSchema, moduleSchema } = opts;

  db.logSQL(`Creating relationships`);
  return Promise.each(relationships, async (rel) => {
    logDebug(
      "[ORADM-FK] Creating FK for field " + rel.tableName + "." + rel.columnName
    );
    if (rel.isCommon && !commonSchema) {
      logDebug(
        "[ORADM-FK] Skipping common FK for field " +
          rel.tableName +
          "." +
          rel.columnName
      );
      return;
    }
    if (!rel.isCommon && !moduleSchema) {
      logDebug(
        "[ORADM-FK] Skipping module FK for field " +
          rel.tableName +
          "." +
          rel.columnName
      );
      return;
    }
    logDebug("[ORADM-FK]", rel);

    // For cross module foreign keys they may be duplicated so drop if they exist
    await db.raw(
      `ALTER TABLE "${rel.schema}"."${rel.tableName}" DROP CONSTRAINT IF EXISTS "${rel.fkName}"`
    );

    let qb = db.schema
      .withSchema(rel.schema)
      .table(rel.tableName, function (table) {
        var fk = table
          .foreign(rel.columnName)
          .references(rel.referenceColumnName)
          .on(`${rel.referenceSchema}.${rel.referenceTableName}`)
          .withKeyName(rel.fkName);

        if (rel.onUpdate != null) fk = fk.onUpdate(rel.onUpdate);
        if (rel.onDelete != null) fk = fk.onDelete(rel.onDelete);
      });
    return db.logSQL(qb);
  });
}

module.exports = createRelationships;
