const mustache = require('mustache');
const Promise = require('bluebird');

const createTablespace = require('./createTablespace');

module.exports = function createSchema(db, schemaName, schemaPassword, opts) {
    var tablespaceName, tempTablespaceName;

    return createDataTablespace()
        .then(createTempTablespace)
        .then(createSchema)
        .then(grantPrivilegesToSchema);

    function createDataTablespace() {
        return createTablespace(db, schemaName, false)
            .then((name) => {
                tablespaceName = name;
            });
    }

    function createTempTablespace() {
        return createTablespace(db, schemaName, true)
            .then((name) => {
                tempTablespaceName = name;
            });
    }

    function createSchema() {
        const createSchemaTemp = `
            CREATE USER "{{schemaName}}" 
                IDENTIFIED BY "{{schemaPassword}}" 
                DEFAULT TABLESPACE {{tablespaceName}}
                TEMPORARY TABLESPACE {{tempTablespaceName}}
        `;

        var sqlStr = mustache.render(createSchemaTemp,
            {
                schemaName,
                schemaPassword,
                tablespaceName,
                tempTablespaceName
            });

        return db.raw(sqlStr);
    }

    function grantPrivilegesToSchema() {
        const grants = [
            'GRANT unlimited tablespace to "{{schemaName}}"',
            'GRANT create session TO "{{schemaName}}"',
            'GRANT create table TO "{{schemaName}}"',
            'GRANT create view TO "{{schemaName}}"',
            'GRANT create any trigger TO "{{schemaName}}"',
            'GRANT create sequence TO "{{schemaName}}"',
            'GRANT CREATE synonym to "{{schemaName}}"'
        ];

        return Promise.each(grants, (grantTemp) => {
            var sqlStr = mustache.render(grantTemp, { schemaName });
            return db.raw(sqlStr);
        });
    }

}