const mustache = require('mustache');
const Promise = require('bluebird');

const detectManagedFileSystem = require('./isManagedFiles');
const doesTablespaceExist = require('./doesTablespaceExist');

const unmanagedTemplate = `
CREATE BIGFILE TABLESPACE {{tablespaceName}}
        DATAFILE '{{tablespaceFilename}}'            
        SIZE 20M                                    
        REUSE                                       
        AUTOEXTEND ON MAXSIZE UNLIMITED                   
        ONLINE`;

const unmanagedTempTemplate = `
CREATE TEMPORARY TABLESPACE {{tablespaceName}}
        TEMPFILE '{{tablespaceFilename}}'       
        SIZE 5M                                    
        REUSE                              
        AUTOEXTEND ON MAXSIZE UNLIMITED`;

const managedTemplate = `CREATE TABLESPACE {{tablespaceName}}`;

const managedTempTemplate = `CREATE TEMPORARY TABLESPACE {{tablespaceName}}`;

module.exports = function createTablespace(db, schemaName, isTemp, opts) {
    var tablespaceName = schemaName + "_DATA";
    if (isTemp) tablespaceName = "TMP_" + tablespaceName;

    var tablespaceFilename = tablespaceName + ".dat";

    return doesTablespaceExist(db, tablespaceName)
           .then((result) => {
               if(result){
                    return Promise.resolve(tablespaceName);
               }else{
                    return detectManagedFileSystem(db)
                            .then((isManaged) => {
                                var sqlStr = "";

                                if (isTemp) {
                                    if (isManaged) {
                                        sqlStr = mustache.render(managedTempTemplate, { tablespaceName, tablespaceFilename });
                                    } else {
                                        sqlStr = mustache.render(unmanagedTempTemplate, { tablespaceName, tablespaceFilename });
                                    }
                                } else {
                                    if (isManaged) {
                                        sqlStr = mustache.render(managedTemplate, { tablespaceName, tablespaceFilename });
                                    } else {
                                        sqlStr = mustache.render(unmanagedTemplate, { tablespaceName, tablespaceFilename });
                                    }
                                }

                                return db.raw(sqlStr)
                                    .then(() => {
                                        return Promise.resolve(tablespaceName);
                                    });
                            });
               }
           });
}

