const Promise = require('bluebird');
const path = require('path');
const _ = require('lodash');
const {shortenName,maxLength} = require('../../../datamodel-csv-parser/Utils');

module.exports = function dropAndRecreateSequences(db, appConfig, opts){

    var knexSpecPath = path.join(appConfig.graphql.dataLibPath, 'db/knex-spec.json');
    var tableSpecs = require(knexSpecPath);

    var fastMode = true;
    if(process.env.recreateSequencesSlowMode){ fastMode = false }
    var log = opts.log || function() {};

    function getSequenceName(name) {
        var formattedName = _.camelCase(name);
        if(formattedName.length > maxLength){
            formattedName = shortenName(name);
        }
        return formattedName.toLowerCase() + "_seq";
    }

    var all = [];

    return Promise.each(tableSpecs.tables, (tableSpec) => {
        var sql = `DROP SEQUENCE "${tableSpec.schema}"."${getSequenceName(tableSpec.tableName)}"`
        all.push(sql);
        if(fastMode) return Promise.resolve();
        log(sql)
        return db.raw(sql).then(function(){ log("OK"); return Promise.resolve(); }).catch(function(){ log("Failed"); return Promise.resolve(); })
    })
    .then(function () {
	    return Promise.each(tableSpecs.tables, (tableSpec) => {
            var sql = `CREATE SEQUENCE "${tableSpec.schema}"."${getSequenceName(tableSpec.tableName)}"  MINVALUE 1 MAXVALUE 9999999999999999999999999999 INCREMENT BY 1 START WITH 1 CACHE 20 NOORDER  NOCYCLE`
            all.push(sql);
            if(fastMode) return Promise.resolve();
            log(sql)
            return db.raw(sql).then(function(){ log("OK"); return Promise.resolve(); }).catch(function(){ log("Failed"); return Promise.resolve(); })
	    })
	})
    .then(function () {
        if(fastMode)
        {
            const cmd =
            `
            BEGIN
            `+all.map(function (sql) {
            return `EXECUTE IMMEDIATE '${sql}';`;
            }).join('\n')+`
            END;
            `
            log(cmd)
            return db.raw(cmd)        
            .then(function () {
                log("Recreate sequences success")
                return Promise.resolve();
            })
            .catch(function (err) {
                log(err)
                log("Batch recreate sequences failed.. trying again one by one (slower)")
                process.env.recreateSequencesSlowMode = true;
                return dropAndRecreateSequences(db, appConfig, opts);
            })
        }
        log("Finished")
        return Promise.resolve();
    })

}