const Promise = require("bluebird");
const fse = require("fs-extra");
const path = require("path");
const _ = require("lodash");

const cleanFolders = require("../utils/cleanFolders");
const datamodelCSVParser = require("../datamodel-csv-parser");
const {
  generateGraphQLSchema,
  convertGraphQLSchemaToString,
  generateKnexSchemaSpec
} = require("../graphql-generator");
const extendGraphQLSchema = require("./extendGraphQLSchema");
const generateDataLib = require("../data-lib-generator");

module.exports = function refreshSchema(appConfig, opts) {
  opts = opts || {};
  var log = opts.log || function() {};

  cleanFolders(appConfig, opts);

  var tableMap = datamodelCSVParser(
    appConfig.graphql.datamodelCSVPath,
    appConfig.graphql.commonDatamodelCSVPath,
    opts
  );

  fse.writeJsonSync(
    path.join(appConfig.graphql.dataLibPath, "db/table-map.json"),
    tableMap,
    { spaces: 2 }
  );

  var knexSpec = generateKnexSchemaSpec(
    tableMap,
    appConfig.db.appSchema,
    appConfig.db.commonSchema,
    appConfig.db.dialect,
    opts
  );
  fse.writeJsonSync(
    path.join(appConfig.graphql.dataLibPath, "db/knex-spec.json"),
    knexSpec,
    { spaces: 2 }
  );

  var schemaSpec = generateGraphQLSchema(tableMap, opts);

  schemaSpec = extendGraphQLSchema(appConfig, schemaSpec, opts);

  fse.writeJsonSync(
    path.join(appConfig.graphql.schemaPath, "schema-spec.json"),
    schemaSpec,
    { spaces: 2 }
  );

  var schemaStr = convertGraphQLSchemaToString(schemaSpec, opts);
  fse.writeFileSync(
    path.join(appConfig.graphql.schemaPath, "schema.gql"),
    schemaStr,
    "utf8"
  );

  var outputFiles = generateDataLib(tableMap, appConfig, opts);
  outputFiles.forEach(genFile => {
    var filePath = genFile.fullPath;
    if (!fse.existsSync(filePath)) {
      if (_.endsWith(filePath, ".json")) {
        fse.outputJsonSync(filePath, genFile.content);
      } else {
        fse.outputFileSync(filePath, genFile.content);
      }
      log("Wrote file to " + filePath);
    } else {
      log("Skipped file " + genFile.relativePath);
    }
  });

  return Promise.resolve();
};
