const { each } = require('lodash');
const njk = require("nunjucks");
const fse = require("fs-extra");
const path = require("path");

const pluralize = require('pluralize');

const entityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./entityTest.njk"),
    "utf8"
);

const initEntitiesTestTemplate = fse.readFileSync(
    path.join(__dirname, "./initEntitiesTest.njk"),
    "utf8"
);

const countEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./countEntityTest.njk"),
    "utf8"
);

const getEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./getEntityTest.njk"),
    "utf8"
);

const queryEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./queryEntityTest.njk"),
    "utf8"
);

const createEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./createEntityTest.njk"),
    "utf8"
);

const updateEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./updateEntityTest.njk"),
    "utf8"
);

const updateEntityQueryTestTemplate = fse.readFileSync(
    path.join(__dirname, "./updateEntityQueryTest.njk"),
    "utf8"
);

const deleteEntityQueryTestTemplate = fse.readFileSync(
    path.join(__dirname, "./deleteEntityQueryTest.njk"),
    "utf8"
);

const deleteEntityTestTemplate = fse.readFileSync(
    path.join(__dirname, "./deleteEntityTest.njk"),
    "utf8"
);

const createNestedRecordsTestTemplate = fse.readFileSync(
    path.join(__dirname, "./createNestedRecordsTest.njk"),
    "utf8"
);

const updateNestedRecordsTestTemplate = fse.readFileSync(
    path.join(__dirname, "./updateNestedRecordsTest.njk"),
    "utf8"
);

const deleteNestedRecordsTestTemplate = fse.readFileSync(
    path.join(__dirname, "./deleteNestedRecordsTest.njk"),
    "utf8"
);

module.exports = function generateDataLibTests(tableMap, appConfig, opts){

    let njkEnv = new njk.Environment(null, {
        trimBlocks: true,
        lstripBlocks: true
    });

    let njkEntityTestTemplate = njk.compile(entityTestTemplate, njkEnv);
    let njkInitEntitiesTestTemplate = njk.compile(initEntitiesTestTemplate, njkEnv);
    
    let njkCountEntityTestTemplate = njk.compile(countEntityTestTemplate, njkEnv);
    let njkGetEntityTestTemplate = njk.compile(getEntityTestTemplate, njkEnv);
    let njkQueryEntityTestTemplate = njk.compile(queryEntityTestTemplate, njkEnv);
    let njkCreateEntityTestTemplate = njk.compile(createEntityTestTemplate, njkEnv);
    let njkUpdateEntityTestTemplate = njk.compile(updateEntityTestTemplate, njkEnv);
    let njkUpdateEntityQueryTestTemplate = njk.compile(updateEntityQueryTestTemplate, njkEnv);
    let njkDeleteEntityQueryTestTemplate = njk.compile(deleteEntityQueryTestTemplate, njkEnv);
    let njkDeleteEntityTestTemplate = njk.compile(deleteEntityTestTemplate, njkEnv);
    
    let njkCreateNestedRecordsTestTemplate = njk.compile(createNestedRecordsTestTemplate, njkEnv);
    let njkUpdateNestedRecordsTestTemplate = njk.compile(updateNestedRecordsTestTemplate, njkEnv);
    let njkDeleteNestedRecordsTestTemplate = njk.compile(deleteNestedRecordsTestTemplate, njkEnv);

    let outputFiles = []

    each(tableMap, (tableDef, entityName) => {

        let content = njkEntityTestTemplate.render({ ...tableDef });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/index.test.js`),
            content
        });

        content = njkCountEntityTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/count${entityName}.test.js`),
            content
        });

        content = njkGetEntityTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/get${entityName}.test.js`),
            content
        });

        content = njkQueryEntityTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/query${entityName}.test.js`),
            content
        });

        content = njkCreateEntityTestTemplate.render({ ...tableDef, entityName, makePlural });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/create${makePlural(entityName)}.test.js`),
            content
        });
       
        content = njkUpdateEntityTestTemplate.render({ ...tableDef, entityName, makePlural });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/update${makePlural(entityName)}.test.js`),
            content
        });
       
        content = njkUpdateEntityQueryTestTemplate.render({ ...tableDef, entityName, makePlural });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/update${makePlural(entityName)}Query.test.js`),
            content
        });

        content = njkDeleteEntityQueryTestTemplate.render({ ...tableDef, entityName, makePlural });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/delete${makePlural(entityName)}Query.test.js`),
            content
        });

        content = njkDeleteEntityTestTemplate.render({ ...tableDef, entityName, makePlural });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/delete${makePlural(entityName)}.test.js`),
            content
        });

        content = njkCreateNestedRecordsTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/createNested${entityName}Records.test.js`),
            content
        });

        content = njkUpdateNestedRecordsTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/updateNested${entityName}Records.test.js`),
            content
        });

        content = njkDeleteNestedRecordsTestTemplate.render({ ...tableDef, entityName });

        outputFiles.push({
            fullPath: path.join(appConfig.graphql.dataLibPath, `entities/${tableDef.modelName}/deleteNested${entityName}Records.test.js`),
            content
        });
    });

    let content = njkInitEntitiesTestTemplate.render({ tableMap });

    outputFiles.push({
        fullPath: path.join(appConfig.graphql.dataLibPath, `entities/index.test.js`),
        content
    });

    return outputFiles;
}


function makePlural(name){
    return pluralize.plural(name);
}
